@@ -22,15 +22,10 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include "file-data.h"
+#include <gio/gio.h>
+#include "glib-utils.h"
 #include "file-utils.h"
-
-#define DESCRIPTION_UNKNOWN _("Unknown type")
-#define DESCRIPTION_SYMLINK _("Symbolic link")
-
-
-static GHashTable *mime_type_hash = NULL;
+#include "file-data.h"
 
 
 FileData *
@@ -39,31 +34,19 @@ file_data_new (void)
 	FileData *fdata;
 
 	fdata = g_new0 (FileData, 1);
-	fdata->mime_type = 0;
+	fdata->content_type = NULL;
 	fdata->free_original_path = FALSE;
 	fdata->dir_size = 0;
 	
-	if (mime_type_hash == NULL)
-		mime_type_hash = g_hash_table_new_full (g_int_hash, 
-							g_int_equal,
-							NULL,
-							(GDestroyNotify) g_free);
-
 	return fdata;
 }
 
 
-void
-file_data_release_data (void)
-{
-	if (mime_type_hash != NULL)
-		g_hash_table_destroy (mime_type_hash);
-}
-
-
 void
 file_data_free (FileData *fdata)
 {
+	if (fdata == NULL)
+		return;
 	if (fdata->free_original_path)
 		g_free (fdata->original_path);
 	g_free (fdata->full_path);
@@ -91,7 +74,7 @@ file_data_copy (FileData *src)
 	fdata->modified = src->modified;
 	fdata->name = g_strdup (src->name);
 	fdata->path = g_strdup (src->path);
-	fdata->mime_type = src->mime_type;
+	fdata->content_type = src->content_type;
 	fdata->encrypted = src->encrypted;
 	fdata->dir = src->dir;
 	fdata->dir_size = src->dir_size;
@@ -115,60 +98,19 @@ file_data_get_type (void)
 }
 
 
-static void
-file_data_update_mime_type (FileData *fdata)
+void
+file_data_update_content_type (FileData *fdata)
 {
-	const char *mime_type;
-
-	mime_type = get_file_mime_type (fdata->full_path, TRUE);
-
-	if (mime_type == NULL) {
-		fdata->mime_type = 0;
-		return;
-	}
-
-	fdata->mime_type = g_str_hash ((gconstpointer) mime_type);
-	if (g_hash_table_lookup (mime_type_hash, (gconstpointer) &fdata->mime_type) == NULL)
-		g_hash_table_insert (mime_type_hash, (gpointer) &fdata->mime_type, g_strdup (mime_type));
-}
-
-
-const char *
-file_data_get_mime_type (const FileData *fdata)
-{
-	const char *mime_type;
-
-	if (fdata->mime_type == 0)
-		file_data_update_mime_type ((FileData*)fdata);
-	if (fdata->mime_type == 0)
-		return GNOME_VFS_MIME_TYPE_UNKNOWN;
-
-	mime_type = g_hash_table_lookup (mime_type_hash, (gconstpointer) &fdata->mime_type);
-	if (mime_type == NULL)
-		mime_type = GNOME_VFS_MIME_TYPE_UNKNOWN;
-
-	return mime_type;
-}
-
-
-const char *
-file_data_get_mime_type_description (const FileData *fdata)
-{
-	const char *desc;
-
-	if (fdata->link != NULL)
-		return DESCRIPTION_SYMLINK;
-
-	desc = gnome_vfs_mime_get_description (file_data_get_mime_type (fdata));
-	if (desc == NULL)
-		desc = DESCRIPTION_UNKNOWN;
-
-	return desc;
+	
+	if (fdata->dir) 
+		fdata->content_type = MIME_TYPE_DIRECTORY;
+	else 
+		fdata->content_type = get_static_string (g_content_type_guess (fdata->full_path, NULL, 0, NULL));
 }
 
 
 gboolean
-file_data_is_dir (const FileData *fdata)
+file_data_is_dir (FileData *fdata)
 {
 	return fdata->dir || fdata->list_dir;
 }
